﻿unit DigitalClockForm;

{
V1.0.0.0  2013-Aug-12  First version
                       Click on the form to exit the clock

V1.0.1.2  2013-Aug-14  Make mouse cursor disappear a few seconds after  ..
                       .. mouse movement stops

V1.0.2.3  2013-Aug-23  Add mouse-wheel control of background brightness
                       You may need to move the mouse first

V1.0.3.4  2013-Aug-27  Save brightness in a .INI file

V1.0.5.7  2014-Apr-20  Look for UTC option in the .INI file

V1.1.0.7  2016-Dec-06  Add "Inverse" option in the .INI file
                       Makes the text white

Nice to add: Honour summer/winter time change  (to be done)



Copyright © David J Taylor, Edinburgh
Web site:  www.satsignal.eu
E-mail:    davidtaylor@writeme.com
}

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, Forms, Controls, Graphics, Dialogs, StdCtrls,
  ExtCtrls, types, Math, IniFiles, Unix;

type

  { TClockForm }

  TClockForm = class (TForm)
    LabelDay: TLabel;
    LabelSS: TLabel;
    LabelHHMM: TLabel;
    LabelDate: TLabel;
    TimerMouseMove: TTimer;
    TimerClock: TTimer;
    procedure FormClick (Sender: TObject);
    procedure FormCreate (Sender: TObject);
    procedure FormDestroy (Sender: TObject);
    procedure FormMouseMove (Sender: TObject; Shift: TShiftState; X, Y: Integer);
    procedure FormMouseWheel (Sender: TObject; Shift: TShiftState;
      WheelDelta: Integer; MousePos: TPoint; var Handled: Boolean);
    procedure FormShow (Sender: TObject);
    procedure TimerClockTimer (Sender: TObject);
    procedure TimerMouseMoveTimer (Sender: TObject);
  private
    { private declarations }
    FBackgroundGrey: integer;
    FIniFile: TIniFile;
    FInverse: boolean;
    FOldHours: Word;
    FOldSeconds: Word;
    FUTC: boolean;
  public
    { public declarations }
  end;

var
  ClockForm: TClockForm;

implementation

{$R *.lfm}

uses
  Video, lazutf8sysutils;

procedure TClockForm.FormClick (Sender: TObject);
begin
  Close;
end;

procedure TClockForm.FormCreate (Sender: TObject);
begin
  // Set to values which will not occur in normal use
  FOldSeconds := 12345;
  FOldHours := 12345;
  FIniFile := TIniFile.Create (ChangeFileExt (Application.ExeName, '.ini'));
  FBackgroundGrey := FIniFile.ReadInteger ('Defaults', 'Brightness', 64);
  Color := RGBToColor (FBackgroundGrey, FBackgroundGrey, FBackgroundGrey);
  FUTC := FIniFile.ReadBool('Defaults', 'UTC', False);
  FInverse := FIniFile.ReadBool('Defaults', 'Inverse', False);
  if FInverse then
    begin
    LabelDay.Font.Color := clWhite;
    LabelSS.Font.Color := clWhite;
    LabelHHMM.Font.Color := clWhite;
    LabelDate.Font.Color := clWhite;
    end;
end;

procedure TClockForm.FormDestroy(Sender: TObject);
begin
  FIniFile.Free;
end;

procedure TClockForm.FormMouseMove (Sender: TObject; Shift: TShiftState; X,
  Y: Integer);
begin
  // Show the normal mouse cursor
  Cursor := crDefault;
  // but run a timer to hide the cursor when the mouse has stopped moving
  TimerMouseMove.Enabled := True;
end;

procedure TClockForm.FormMouseWheel (Sender: TObject; Shift: TShiftState;
  WheelDelta: Integer; MousePos: TPoint; var Handled: Boolean);
begin
  if WheelDelta > 0
  then Inc (FBackgroundGrey, 8)
  else
    if WheelDelta < 0 then Dec (FBackgroundGrey, 8);
  FBackgroundGrey := Max (FBackgroundGrey, 8);
  FBackgroundGrey := Min (FBackgroundGrey, 255);
  FIniFile.WriteInteger ('Defaults', 'Brightness', FBackgroundGrey);
  Color := RGBToColor (FBackgroundGrey, FBackgroundGrey, FBackgroundGrey);
end;

procedure TClockForm.FormShow (Sender: TObject);
var
  w: integer;
  f: integer;
begin
  // Set the font size for the labels
  LabelHHMM.Caption := FormatDateTime ('hh:mm', Now);
  repeat
    LabelHHMM.Font.Size := LabelHHMM.Font.Size - 3
  until LabelHHMM.Canvas.TextHeight('88') > 7 * ClientHeight div 20;
  f := LabelHHMM.Font.Size;
  if FUTC then
    begin
    LabelSS.Caption := LabelSS.Caption + ' utc';
    LabelHHMM.Font.Size := 4 * LabelHHMM.Font.Size div 5;
    end;
  LabelSS.Font.Size := 5 * LabelHHMM.Font.Size div 7;
  LabelDay.Font.Size := f;
  LabelDate.Font.Size := 5 * f div 7;
  LabelSS.Top := LabelHHMM.Top + LabelHHMM.Height - 107 * LabelSS.Height div 100;
  w := LabelHHMM.Width + LabelSS.Width;
  LabelHHMM.Left := (ClientWidth - w) div 2;
  LabelSS.Left := LabelHHMM.Left + LabelHHMM.Width;
  LabelDate.Top := ClientHeight - 10 - LabelDate.Height;
  LabelDate.Left := (ClientWidth - LabelDate.Width) div 2;
  LabelDay.Left := (ClientWidth - LabelDay.Width) div 2;
  LabelDay.Top := LabelDate.Top - 10 - LabelDay.Height;
end;

procedure TClockForm.TimerClockTimer (Sender: TObject);
var
  when: TDateTime;
  hours, minutes, seconds, th: Word;
begin
  if FUTC
    then when := NowUTC
    else when := Now;
  DecodeTime (when, hours, minutes, seconds, th);
  if seconds <> FOldSeconds then
    begin
    FOldSeconds := seconds;
    LabelHHMM.Caption := FormatDateTime ('hh:mm', when);
    LabelHHMM.Update;
    LabelSS.Caption := FormatDateTime (' ss', when);
    if FUTC then LabelSS.Caption := LabelSS.Caption + ' utc';
    LabelSS.Update;
    if hours <> FOldHours then
      begin
//      ReReadLocalTime;      // Check whether time offset has changed
       FOldHours := hours;
      LabelDate.Caption := FormatDateTime ('yyyy-mmm-dd', when);
      LabelDay.Caption := FormatDateTime ('dddd', when);
      LabelDate.Left := (ClientWidth - LabelDate.Width) div 2;
      LabelDay.Left := (ClientWidth - LabelDay.Width) div 2;
      end;
    end;
end;

procedure TClockForm.TimerMouseMoveTimer (Sender: TObject);
begin
  // Hide the mouse cursor and stop the timer
  TimerMouseMove.Enabled := False;
  Cursor := crNone;
end;

end.
